/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2008-2013 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package org.glassfish.grizzly.samples.echo;

import static org.junit.Assert.fail;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;
import java.nio.charset.Charset;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import org.glassfish.grizzly.Connection;
import org.glassfish.grizzly.filterchain.FilterChainBuilder;
import org.glassfish.grizzly.filterchain.TransportFilter;
import org.glassfish.grizzly.nio.transport.TCPNIOTransport;
import org.glassfish.grizzly.nio.transport.TCPNIOTransportBuilder;
import org.glassfish.grizzly.utils.Charsets;
import org.glassfish.grizzly.utils.StringFilter;

/**
 * The simple client, which sends a message to the echo server
 * and waits for response
 * @author Alexey Stashok
 */
public class EchoClient extends Thread{

	public Connection connection = null;
//	 final TCPNIOTransport transport;
	 Socket plainSocket;
	 
    public EchoClient()	throws IOException,
            ExecutionException, InterruptedException, TimeoutException {



//        // Create a FilterChain using FilterChainBuilder
//        FilterChainBuilder filterChainBuilder = FilterChainBuilder.stateless();
//        // Add TransportFilter, which is responsible
//        // for reading and writing data to the connection
//        filterChainBuilder.add(new TransportFilter());
//        // StringFilter is responsible for Buffer <-> String conversion
//        filterChainBuilder.add(new StringFilter(Charset.forName("UTF-8")));
//        // ClientFilter is responsible for redirecting server responses to the standard output
//        filterChainBuilder.add(new ClientFilter());
//
//        // Create TCP transport
//        transport =
//                TCPNIOTransportBuilder.newInstance().build();
//        transport.setProcessor(filterChainBuilder.build());
//        // start the transport
//        transport.start();
//
//        // perform async. connect to the server
//        Future<Connection> future = transport.connect(EchoServer.HOST,
//                EchoServer.PORT);
//        // wait for connect operation to complete
//        connection = future.get(10, TimeUnit.SECONDS);
//
//        assert connection != null;
    	openPlainConnection();
    }
    
    private void openPlainConnection()
	{
    	plainSocket = new Socket();
		
		try
		{
			//open connection
			InetAddress hostAddress = InetAddress.getByName(EchoServer.HOST);
			SocketAddress socketAddress = new InetSocketAddress(hostAddress, EchoServer.PORT);
			
			// connect to plain port
			plainSocket.connect(socketAddress, 10000);
		}
		catch (IOException e) 
		{
			e.printStackTrace();
			fail(e.getMessage());
		}		
	}
    
    public void writeString(String str) throws IOException
    {
//    	connection.write(str);
		BufferedWriter wr = new BufferedWriter(new OutputStreamWriter(plainSocket.getOutputStream()));
		wr.write(str+"\r\n");
		wr.flush();
		
		BufferedReader rd = new BufferedReader(new InputStreamReader(plainSocket.getInputStream()));
		String line = rd.readLine();
		System.out.println("Got the followong from server: [" +line + "]");


    }
    
    public void stopClient()
    {
    	synchronized (this) {
			notify();
		}
    } 
    
    @Override
   	public void run() {
    	   try {

               synchronized (this) {
   				wait();
               }
               
//               System.out.println("Ready... (\"q\" to exit)");
//               final BufferedReader inReader = new BufferedReader(
//                       new InputStreamReader(System.in, Charsets.ASCII_CHARSET));
//               do {
//                   final String userInput = inReader.readLine();
//                   if (userInput == null || "q".equals(userInput)) {
//                       break;
//                   }
   //
//                   connection.write(userInput);
//               } while (true);
         
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
               // close the client connection
//               if (connection != null) {
//                   connection.closeSilently();
//               }
//
//               // stop the transport
//               try {
//				transport.shutdownNow();
//			} catch (IOException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			}
			try {
				plainSocket.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
           }
   	}
}
